#pragma once

/*
based on code of Copyright (c) Martin Kinkelin
*/
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "../WinRing0/OlsApi.h"


/// <summary>
/// Manages the Turbo/hardware boost feature of newer CPUs.
/// </summary>
namespace Npstates{
class TurboManager
{
public:

	/// <summary>Returns true if the CPU supports Turbo.</summary>
	static bool IsSupported()
	{
		// cached
		static int result = -1;
		if (result == -1)
		{
			DWORD eax, ebx, ecx, edx;
			if (Cpuid(0x80000007u, &eax, &ebx, &ecx, &edx))
				result = ((edx >> 9) & 1);
			else
				result = 0;
		}

		return (result != 0);
	}


	/// <summary>Returns the number of boosted (Turbo) P-states.</summary>
	static ureg32 GetNumBoostedStates()
	{
		if (!IsSupported())
			return 0;

		DWORD lower;
		if ( !ReadPciConfigDwordEx(0xC4, 0x15C, &lower) )return 0;
			//return 1; // assume it's enabled by default if supported

		return (lower >> 2) & 7;
	}
};

//namespace Npstates
}